// お習字ロボットアーム

#include <SPI.h>
#include <Adb.h>
#include <Servo.h>

// 角度[deg]コマンド
#define COMMAND_DEGREE    0x5A
// パルス幅[usec]コマンド
#define COMMAND_MICROSEC  0x55

// 各サーボの角度補正 (機械の向き、取り付け角度の誤差を補正)
#define SERVO_CORRECT0(x) (180 - (x))
#define SERVO_CORRECT1(x) (180 - (x+5))
#define SERVO_CORRECT2(x) (x)
#define SERVO_CORRECT3(x) (x)

// 各サーボのパルス幅補正 (機械の向き、取り付け角度の誤差を補正)
#define PULSE_CORRECT0(x) (3000 - (x))
#define PULSE_CORRECT1(x) (3000 - (x+50))
#define PULSE_CORRECT2(x) (x)
#define PULSE_CORRECT3(x) (x)


// パルス幅範囲クリップ (600us〜2400us)
#define PULSE_CLIP(x) ((x<600) ? 600 : ((x>2400) ? 2400 : x) )

// ラジコンサーボ 4自由度
// [0]肩ヨー軸 [1]肩ピッチ軸 [2]肘ピッチ軸 [3]手首ピッチ軸
Servo servos[4];

// ADB接続 (Androidとの接続)
Connection * connection;

// ADB接続のイベントハンドラ
void adbEventHandler(Connection * connection, adb_eventType event, uint16_t length, uint8_t * data)
{
  unsigned int t[4];

  // 受信イベントか？
  if (event == ADB_CONNECTION_RECEIVE)
  {
    // 角度[deg]コマンドの場合
    if(data[0] == COMMAND_DEGREE)
    {
      // 各サーボの角度指令
      servos[0].write(SERVO_CORRECT0(data[1]));
      servos[1].write(SERVO_CORRECT1(data[2]));
      servos[2].write(SERVO_CORRECT2(data[3]));
      servos[3].write(SERVO_CORRECT3(data[4]));
    }
    // パルス幅[usec]コマンドの場合
    else if(data[0] == COMMAND_MICROSEC)
    {
      // 各サーボのパルス幅を解釈
      t[0] = (((unsigned int)data[1]&0xFF)<<8) + ((unsigned int)data[2]&0xFF); 
      t[1] = (((unsigned int)data[3]&0xFF)<<8) + ((unsigned int)data[4]&0xFF); 
      t[2] = (((unsigned int)data[5]&0xFF)<<8) + ((unsigned int)data[6]&0xFF); 
      t[3] = (((unsigned int)data[7]&0xFF)<<8) + ((unsigned int)data[8]&0xFF);
      t[0] = PULSE_CLIP(t[0]);
      t[1] = PULSE_CLIP(t[1]);
      t[2] = PULSE_CLIP(t[2]);
      t[3] = PULSE_CLIP(t[3]);
      if(t[1]<1200){
        t[1] += ((1200-t[1])*30)/300;
      }
      t[0] = PULSE_CORRECT0(t[0]);
      t[1] = PULSE_CORRECT1(t[1]);
      t[2] = PULSE_CORRECT2(t[2]);
      t[3] = PULSE_CORRECT3(t[3]);
      
      // 各サーボのパルス幅指令
      servos[0].writeMicroseconds(t[0]);
      servos[1].writeMicroseconds(t[1]);
      servos[2].writeMicroseconds(t[2]);
      servos[3].writeMicroseconds(t[3]);      
    }
  }
}

// 初期化
void setup()
{
  // Initialise serial port 必要？
  Serial.begin(57600);
  
  // サーボのピン設定
  // HiTEC HSR-8498HBは、0deg:600us 90deg:1500us 180deg:2400us
  servos[0].attach(6,600,2400); // Digital6:肩ヨー軸
  servos[1].attach(5,600,2400); // Digital5:肩ピッチ軸
  servos[2].attach(4,600,2400); // Digital4:肘ピッチ軸
  servos[3].attach(3,600,2400); // Digital3:手首ピッチ軸

  // サーボの角度初期値
  servos[0].write(SERVO_CORRECT0(90));
  servos[1].write(SERVO_CORRECT1(70));
  servos[2].write(SERVO_CORRECT2(60));
  servos[3].write(SERVO_CORRECT3(60));
//  servos[0].writeMicroseconds(PULSE_CORRECT0(1500));
//  servos[1].writeMicroseconds(PULSE_CORRECT1(600+50*10 + 16));
//  servos[2].writeMicroseconds(PULSE_CORRECT2(600+80*10));
//  servos[3].writeMicroseconds(PULSE_CORRECT3(600+(180-50)*10));

  // ADBの初期化  
  ADB::init();

  // ADB接続を開く
  // TCPのポート4567、自動再接続有効、イベントハンドラの登録
  connection = ADB::addConnection("tcp:4567", true, adbEventHandler);  
}

// メインループ
void loop()
{
  // ADBのポーリング
  ADB::poll();
}

